//
//  Brain.m
//  Gyproc
//
//  Created by Bert Carrein on 14/06/11.
//  Copyright 2011 DMS. All rights reserved.
//

#import "Brain.h"


@implementation Brain

@synthesize toepassingCategorieArray;
@synthesize productCategorieArray;
@synthesize productToepassingen;
@synthesize typeArray;
@synthesize managedObjectContext;
@synthesize currentLang;

static Brain *brain;

//Taal wijzigen & opslaan in voorkeuren
- (void)setCurrentLang:(NSString *)aCurrentLang {
    currentLang = [aCurrentLang retain];
    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    [prefs setObject:aCurrentLang forKey:@"lang"];
    [prefs synchronize];
}

//Alle objecten van een bepaald type wissen
- (void)deleteAllObjects:(NSString *)entityDescription {
    NSFetchRequest *fetchRequest = [[NSFetchRequest alloc] init];
    NSEntityDescription *entity = [NSEntityDescription entityForName:entityDescription inManagedObjectContext:managedObjectContext];
    [fetchRequest setEntity:entity];
    
    NSError *error;
    NSArray *items = [managedObjectContext executeFetchRequest:fetchRequest error:&error];
    [fetchRequest release];
    
    for (NSManagedObject *managedObject in items) {
        [managedObjectContext deleteObject:managedObject];
    }
    
    [managedObjectContext save:&error];
}

//Toepassingen, stappen , wistjedats, toepassingcategorieen uit plist uitlezen en invoegen in db
- (void)loadToepassingDataFromPropertyList:(NSString *)entity {
    NSString *path = [[NSBundle mainBundle] pathForResource:entity ofType:@"plist"];
    NSArray *items = [NSMutableArray arrayWithContentsOfFile:path];
    
    int categorieTeller = 0;
    int toepassingTeller = 0;
    for (NSDictionary *dict in items) {
        if ([entity isEqualToString:@"Toepassing"]) {
            for (ToepassingenCategorie *cat in self.toepassingCategorieArray) {
                if ([cat.titelNL isEqualToString:[dict objectForKey:@"categorie"]]) {
                    toepassingTeller ++;
                    NSManagedObject *m = [NSEntityDescription insertNewObjectForEntityForName:entity inManagedObjectContext:managedObjectContext];
                    [m setValuesForKeysWithDictionary:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:toepassingTeller], @"toepassingID", [dict objectForKey:@"titelNL"], @"titelNL", [dict objectForKey:@"titelFR"], @"titelFR", [dict objectForKey:@"omschrijvingNL"], @"omschrijvingNL", [dict objectForKey:@"omschrijvingFR"], @"omschrijvingFR", [dict objectForKey:@"videoURLNL"], @"videoURLNL", [dict objectForKey:@"videoURLFR"], @"videoURLFR", [dict objectForKey:@"imageURL"], @"imageURL", [dict objectForKey:@"stapImageURL"], @"stapImageURL", cat, @"toepassingenCategorie", nil]];
                }
            }
        } else if ([entity isEqualToString:@"Stap"]) {
            for (ToepassingenCategorie *cat in self.toepassingCategorieArray) {
                for (Toepassing *toep in cat.toepassingen) {
                    if ([toep.stapImageURL isEqualToString:[dict objectForKey:@"toepassing"]]) {
                        NSManagedObject *m = [NSEntityDescription insertNewObjectForEntityForName:entity inManagedObjectContext:managedObjectContext];
                        [m setValuesForKeysWithDictionary:[NSDictionary dictionaryWithObjectsAndKeys:[dict objectForKey:@"stap"], @"stap", [dict objectForKey:@"tekstNL"], @"tekstNL", [dict objectForKey:@"tekstFR"], @"tekstFR", [dict objectForKey:@"tipNL"], @"tipNL", [dict objectForKey:@"tipFR"], @"tipFR", toep, @"toepassing", nil]];
                    }
                }
            }
        } else if ([entity isEqualToString:@"Wistjedat"]) {
            for (ToepassingenCategorie *cat in self.toepassingCategorieArray) {
                for (Toepassing *toep in cat.toepassingen) {
                    if ([toep.stapImageURL isEqualToString:[dict objectForKey:@"toepassing"]]) {
                        NSManagedObject *m = [NSEntityDescription insertNewObjectForEntityForName:entity inManagedObjectContext:managedObjectContext];
                        [m setValuesForKeysWithDictionary:[NSDictionary dictionaryWithObjectsAndKeys:[dict objectForKey:@"wjdNL"], @"wjdNL", [dict objectForKey:@"wjdFR"], @"wjdFR", toep, @"toepassing", nil]];
                    }
                }
            }
        } else if ([entity isEqualToString:@"ToepassingenCategorie"]) {
            categorieTeller ++;
            NSManagedObject *m = [NSEntityDescription insertNewObjectForEntityForName:entity inManagedObjectContext:managedObjectContext];
            [m setValuesForKeysWithDictionary:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:categorieTeller], @"categorieID", [dict objectForKey:@"titelNL"], @"titelNL", [dict objectForKey:@"titelFR"], @"titelFR", nil]];
        }
    }
    
    NSError *err = nil;
    [managedObjectContext save:&err];
}

//ToepassingenCategorie objecten ophalen uit database, gesorteerd op categorieID
- (void)fetchToepassingRecords {
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"ToepassingenCategorie" inManagedObjectContext:managedObjectContext];
    
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entity];
    
    NSSortDescriptor *sortDescriptor  = [[NSSortDescriptor alloc] initWithKey:@"categorieID" ascending:YES];
    NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    [request setSortDescriptors:sortDescriptors];
    [sortDescriptor release];
    
    NSError *error;
    NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    
    self.toepassingCategorieArray = mutableFetchResults;
    [mutableFetchResults release];
    [request release];
}

//Producten ophalen uit productcataloog en database ermee vullen (Product & ProductenCategorie)
- (void)loadProductDataFromPropertyList {
    NSString *path = [[NSBundle mainBundle] pathForResource:@"product_cataloog" ofType:@"plist"];
    NSArray *items = [NSMutableArray arrayWithContentsOfFile:path];
    
    for (NSDictionary *dict in items) {
        NSMutableSet *products = [[NSMutableSet alloc] init];
        
        NSArray *oldProducts = (NSArray *)[dict objectForKey:@"products"];
        for (NSDictionary *product in oldProducts) {
            
            Product *prod = [[Product alloc] initWithEntity:[NSEntityDescription entityForName:@"Product" inManagedObjectContext:managedObjectContext] insertIntoManagedObjectContext:managedObjectContext];
            
            prod.nodename = [product objectForKey:@"nodename"];
            prod.productpicture = [product objectForKey:@"productpicture"];
            prod.titelNL = [product objectForKey:@"titleNL"];
            prod.titelFR = [product objectForKey:@"titleFR"];
            prod.descriptionNL = [product objectForKey:@"descriptionNL"];
            prod.descriptionFR = [product objectForKey:@"descriptionFR"];
            prod.productID = [product objectForKey:@"id"];
            
            [products addObject:prod];
            [prod release];
        }
        
        ProductenCategorie *categorie = [[ProductenCategorie alloc] initWithEntity:[NSEntityDescription entityForName:@"ProductenCategorie" inManagedObjectContext:managedObjectContext] insertIntoManagedObjectContext:managedObjectContext];
        
        categorie.nodename = [dict objectForKey:@"nodename"];
        categorie.steppicture = [dict objectForKey:@"steppicture"];
        categorie.titelNL = [dict objectForKey:@"titleNL"];
        categorie.titelFR = [dict objectForKey:@"titleFR"];
        categorie.shortDescriptionNL = [dict objectForKey:@"shortdescriptionNL"];
        categorie.shortDescriptionFR = [dict objectForKey:@"shortdescriptionFR"];
        categorie.products = products;
        categorie.categorieID = [dict objectForKey:@"id"];
        
        [categorie autorelease];
        [products release];
    }
    
    NSError *err = nil;
    [managedObjectContext save:&err];
}

//Aftmetingen ophalen uit types.plist & in database opvullen
- (void)loadTypesFromPropertyList {
    NSString *path = [[NSBundle mainBundle] pathForResource:@"types" ofType:@"plist"];
    NSArray *items = [NSMutableArray arrayWithContentsOfFile:path];
    
    for (NSDictionary *dict in items) {
        Afmeting *afmeting = [[Afmeting alloc] initWithEntity:[NSEntityDescription entityForName:@"Afmetingen" inManagedObjectContext:managedObjectContext] insertIntoManagedObjectContext:managedObjectContext];
        
        afmeting.code = [dict objectForKey:@"id"];
        afmeting.descriptionNL = [dict objectForKey:@"meervoudNL"];
        afmeting.descriptionFR = [dict objectForKey:@"meervoudFR"];
        afmeting.descriptionEnkelvoudNL = [dict objectForKey:@"enkelvoudNL"];
        afmeting.descriptionEnkelvoudFR = [dict objectForKey:@"enkelvoudFR"];
        
        [afmeting autorelease];
    }
    
    NSError *err = nil;
    [managedObjectContext save:&err];
}

//Afmetingen ophalen uit database
- (void)fetchTypes {
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"Afmetingen" inManagedObjectContext:managedObjectContext];
    
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entity];
    
    NSError *error;
    NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    
    self.typeArray = mutableFetchResults;
    [mutableFetchResults release];
    [request release];
}

//Objecten van type ProductenCategorie ophalen uit database en sorteren op categorieID
- (void)fetchRecords {
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"ProductenCategorie" inManagedObjectContext:managedObjectContext];
    
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entity];
    
    NSSortDescriptor *sortDescriptor  = [[NSSortDescriptor alloc] initWithKey:@"categorieID" ascending:YES];
    NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    [request setSortDescriptors:sortDescriptors];
    [sortDescriptor release];
    
    NSError *error;
    NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    
    self.productCategorieArray = mutableFetchResults;
    [mutableFetchResults release];
    [request release];
}

//Excelfiles uitlezen en objecten van het type ProductToepassing & productInformatie ermee opvullen
- (void)getExcelDataForBerekeningenCategorie:(BerekeningenCategorie *)categorie {
    NSBundle *bundle = [NSBundle mainBundle];
	NSString *textFilePath = [bundle pathForResource:categorie.fileNL ofType:@"csv"];
	NSError *error;
	NSString *fileContents = [[NSString alloc] initWithContentsOfFile:textFilePath encoding:NSStringEncodingConversionAllowLossy error:&error];
    
    NSMutableArray *rijen = [[NSMutableArray alloc] init];
    
    NSArray *lijnen = [fileContents componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]];
    for (NSString *lijn in lijnen) {
        NSArray *cellen = [lijn componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@";"]];
        [rijen addObject:cellen];
    }
    [fileContents release];
    
    ProductToepassing *toepassing = [[ProductToepassing alloc] init];
    toepassing.titelNL = categorie.titelNL;
    toepassing.titelFR = categorie.titelFR;
    toepassing.descriptionNL = categorie.descriptionNL;
    toepassing.descriptionFR = categorie.descriptionFR;
    toepassing.tekstNL = categorie.selectTextNL;
    toepassing.tekstFR = categorie.selectTextFR;
    toepassing.pic = categorie.pic;
    toepassing.productInformaties = [[NSMutableArray alloc] init];
    
    ProductInformatie *info = nil;
    NSMutableArray *pickerviewData = [[NSMutableArray alloc] init];
    int row = 0;
    NSString *nextInformation = nil;
    
    NSMutableArray *eenhedenPerLengte;
    NSMutableArray *hoeveelhedenPerLengte;
    
    for (int rij = 1; rij <= [rijen count]; rij++) {
        NSArray *rijArr = [[rijen objectAtIndex:rij-1] retain];
        
        for (int kol = 1; kol <= [rijArr count]; kol++) {
            NSString *inhoud = [[rijArr objectAtIndex:kol-1] retain];
            
            //pickerView data ophalen
            if (rij == 2) {
                if (![inhoud isEqualToString:@""]) {
                    [pickerviewData addObject:inhoud];
                }
            }
            
            //productinformatie data ophalen
            if (kol == 1 && info == nil && ![inhoud isEqualToString:@""] && ![inhoud isEqualToString:@"0"]) {
                for (ProductenCategorie *categorie in self.productCategorieArray) {
                    for (Product *product in categorie.products) {
                        if ([product.productID intValue] == [inhoud intValue]) {
                            //Nieuw ProductInformatie object aanmaken
                            row = 1;
                            [info release];
                            info = [[ProductInformatie alloc] init];
                            info.product = product;
                            info.extraInfo = nextInformation;
                            nextInformation = nil;
                            
                            eenhedenPerLengte = [NSMutableArray array];
                            hoeveelhedenPerLengte = [NSMutableArray array];
                        }
                    }
                }
            } else if (row == 2 && info != nil && kol != 1) {
                if (kol >= 5) {
                    [hoeveelhedenPerLengte addObject:inhoud];
                } else if (kol == 2) {
                    info.typeHoeveelheid = inhoud;
                }
                
                if (kol==[rijArr count]) {
                    info.eenhedenPerLengte = eenhedenPerLengte;
                    info.hoeveelhedenPerLengte = hoeveelhedenPerLengte;
                    [toepassing.productInformaties addObject:info];
                    info = nil;
                    row = 0;
                }
            } else if (row == 1 && info != nil && kol == [rijArr count]) {
                row = 2;
                [eenhedenPerLengte addObject:inhoud];
            } else if (row == 1 && info != nil && kol != 1) {
                if (kol >= 5) {
                    [eenhedenPerLengte addObject:inhoud];
                } else if (kol == 2) {
                    info.typeEenheid = inhoud;
                } else if (kol == 3) {
                    if ([inhoud isEqualToString:@"or"]) {
                        info.of = YES;
                    }
                }
            } else if (kol == 1 && info != nil && ([inhoud isEqualToString:@""] || [inhoud isEqualToString:@"0"])) {
                info.eenhedenPerLengte = eenhedenPerLengte;
                info.hoeveelhedenPerLengte = hoeveelhedenPerLengte;
                [toepassing.productInformaties addObject:info];
                info = nil;
                row = 0;
            }
            
            if (kol == 1 && [inhoud isEqualToString:@"0"]) {
                nextInformation = [rijArr objectAtIndex:1];
            }
            [inhoud release];
        }
        [rijArr release];
    }
    [rijen release];
    [info release];
    
    toepassing.pickerViewData = pickerviewData;
    [pickerviewData release];
    
    [productToepassingen addObject:toepassing];
    [toepassing release];
}

//Berekeningen (overeenkomend met excelfiles) ophalen uit plist
- (void)loadBerekeningCategorieenFromPropertyList {
    NSString *path = [[NSBundle mainBundle] pathForResource:@"BerekeningenCategorie" ofType:@"plist"];
    NSArray *items = [NSMutableArray arrayWithContentsOfFile:path];
    
    int categorieTeller = 0;
    for (NSDictionary *dict in items) {
        categorieTeller ++;
        NSManagedObject *m = [NSEntityDescription insertNewObjectForEntityForName:@"BerekeningenCategorie" inManagedObjectContext:managedObjectContext];
        [m setValuesForKeysWithDictionary:[NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:categorieTeller], @"categorieID", [dict objectForKey:@"titleNL"], @"titelNL", [dict objectForKey:@"titleFR"], @"titelFR", [dict objectForKey:@"descriptionNL"], @"descriptionNL", [dict objectForKey:@"descriptionFR"], @"descriptionFR", [dict objectForKey:@"selectTextNL"], @"selectTextNL", [dict objectForKey:@"selectTextFR"], @"selectTextFR", [dict objectForKey:@"fileNL"], @"fileNL", [dict objectForKey:@"pic"], @"pic", nil]];
    }
    
    NSError *err = nil;
    [managedObjectContext save:&err];
}

//Objecten van type BerekeningenCategorie ophalen uit db en sorteren op categorieID
- (void)fetchBerekeningCategorieen {
    NSEntityDescription *entity = [NSEntityDescription entityForName:@"BerekeningenCategorie" inManagedObjectContext:managedObjectContext];
    NSFetchRequest *request = [[NSFetchRequest alloc] init];
    [request setEntity:entity];
    
    NSSortDescriptor *sortDescriptor  = [[NSSortDescriptor alloc] initWithKey:@"categorieID" ascending:YES];
    NSArray *sortDescriptors = [NSArray arrayWithObject:sortDescriptor];
    [request setSortDescriptors:sortDescriptors];
    [sortDescriptor release];
    
    NSError *error;
    NSMutableArray *mutableFetchResults = [[managedObjectContext executeFetchRequest:request error:&error] mutableCopy];
    [request release];
    
    productToepassingen = [[NSMutableArray alloc] init];
    for (BerekeningenCategorie *cat in mutableFetchResults) {
        [self getExcelDataForBerekeningenCategorie:cat];
    }
    [mutableFetchResults release];
}

//Database vullen met data en data ophalen uit database
- (void)getDataAndFiles {
    [self deleteAllObjects:@"ToepassingenCategorie"];
    [self deleteAllObjects:@"Product"];
    [self deleteAllObjects:@"Toepassing"];
    [self deleteAllObjects:@"Stap"];
    [self deleteAllObjects:@"Wistjedat"];
    [self deleteAllObjects:@"ProductenCategorie"];
    [self deleteAllObjects:@"BerekeningenCategorie"];
    [self deleteAllObjects:@"Afmetingen"];
    
    [self loadToepassingDataFromPropertyList:@"ToepassingenCategorie"];
    [self fetchToepassingRecords];
    
    [self loadToepassingDataFromPropertyList:@"Toepassing"];
    [self fetchToepassingRecords];
    
    [self loadToepassingDataFromPropertyList:@"Stap"];
    [self fetchToepassingRecords];
    
    [self loadToepassingDataFromPropertyList:@"Wistjedat"];
    [self fetchToepassingRecords];
    
    for (ToepassingenCategorie *cat in self.toepassingCategorieArray) {
        cat.sortedToepassingen = [cat.toepassingen sortedArrayUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"toepassingID" ascending:YES]]];
    }
    
    [self loadProductDataFromPropertyList];
    [self fetchRecords];
    
    for (ProductenCategorie *cat in self.productCategorieArray) {
        cat.sortedProducts = [cat.products sortedArrayUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"productID" ascending:YES]]];
    }
    
    [self loadBerekeningCategorieenFromPropertyList];
    [self fetchBerekeningCategorieen];
    
    [self loadTypesFromPropertyList];
    [self fetchTypes];
}

//Data ophalen uit database
//Deze functie wordt opgeroepen als de applicatie al eens is opgestart. De data moet dus niet opnieuw in de database worden gestopt
- (void)getData {
    [self fetchToepassingRecords];
    for (ToepassingenCategorie *cat in self.toepassingCategorieArray) {
        cat.sortedToepassingen = [cat.toepassingen sortedArrayUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"toepassingID" ascending:YES]]];
    }
    
    [self fetchRecords];
    for (ProductenCategorie *cat in self.productCategorieArray) {
        cat.sortedProducts = [cat.products sortedArrayUsingDescriptors:[NSArray arrayWithObject:[NSSortDescriptor sortDescriptorWithKey:@"productID" ascending:YES]]];
    }
    
    [self fetchBerekeningCategorieen];
    [self fetchTypes];
}

//Opvragen van een brain object. Eenmaal er een object werd aangemaakt wordt altijd met dezelfde gewerkt
+ (Brain *)brain {
    if (brain == nil) {
        brain = [[Brain alloc] init];
        return brain;
    }
    return brain;
}

//Opvragen van een brain object. Eenmaal er een object werd aangemaakt wordt altijd met dezelfde gewerkt
//Deze functie moet worden opgeroepen als het brain voor de eerste maal aangemaakt: er is een managed object model nodig
+ (Brain *)brainForManagedObjectContext:(NSManagedObjectContext *)moc {
    if (brain == nil) {
        brain = [[Brain alloc] init];
        brain.managedObjectContext = moc;
        return brain;
    }
    return brain;
}

- (void)dealloc {
    [toepassingCategorieArray release];
    [productCategorieArray release];
    [productToepassingen release];
    [typeArray release];
    [managedObjectContext release];
    [currentLang release];
    
    [super dealloc];
}

@end
