//
//  FotoView.m
//  Gyproc
//
//  Created by Bert Carrein on 17/05/11.
//  Copyright 2011 DMS. All rights reserved.
//

#import "FotoView.h"


@implementation FotoView

@synthesize delegate;

//Images tonen adhv lazy loading
- (void)showImages:(int)newStap {
    for (UIImageView *img in [[scrollImage.subviews copy] autorelease]) {
        if ([img isKindOfClass:[UIImageView class]]) {
            for (Stap *stap in stappen) {
                if ([stap.stap intValue] == img.tag) {
                    if (img.tag >= newStap-2 && img.tag <= newStap+2) {
                        if (!img.image) {
                            UIImage *image = [[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[NSString stringWithFormat:@"%@_%i", stap.toepassing.stapImageURL, img.tag] ofType:@"jpg"]];
                            img.image = image;
                            [image release];
                        }
                        img.hidden = NO; 
                    } else {
                        img.hidden = YES;
                    }
                }
            }
        }
    }
}

//Terugkeren naar StartView Controller (animatie)
- (void)dismissController {
    [delegate dismissFotoView:self];
}
- (void)goBack:(UIGestureRecognizer *)sender {
    if ([sender isKindOfClass:[UITapGestureRecognizer class]] || [sender isKindOfClass:[UIPinchGestureRecognizer class]] && [(UIPinchGestureRecognizer *)sender scale] < 1) {
        scrollImage.userInteractionEnabled = NO;
        [delegate setStapForTag:sender.view.tag animated:YES];
        if ([startOrientation isEqualToString:currentOrientation]) {
            float xPos = 412;
            float yPos = 244;
            if (UIInterfaceOrientationIsPortrait(self.interfaceOrientation)) {
                xPos = 20;
                yPos = 465;
            }
            
            float width = oldImage.frame.size.width;
            float height = sender.view.frame.size.height/sender.view.frame.size.width*width;
            
            float xCenterNew = (sender.view.tag-1) * scrollImage.frame.size.width + oldImage.frame.size.width/2 - width/2 + xPos;
            float yCenterNew = oldImage.frame.size.height/2 - height/2 + yPos;
            
            [UIView beginAnimations:nil context:nil];
            [UIView setAnimationDuration:0.5];
            sender.view.frame = CGRectMake(xCenterNew, yCenterNew, width, height);
            self.view.backgroundColor = [UIColor colorWithRed:0 green:0 blue:0 alpha:0];
            [UIView commitAnimations];
            
            [self performSelector:@selector(dismissController) withObject:nil afterDelay:0.5];
        } else {
            [self dismissController];
        }
    }
}

//Stap meegeven die bij de tag hoort
- (Stap *)getStapForTag:(int)tag {
    for (Stap *stap in stappen) {
        if ([stap.stap intValue] == tag) {
            return stap;
        }
    }
    return nil;
}

//Gesture recognizers toevoegen (zoom gesture & double tap)
- (void)addGestureRecognizer:(UIView *)view {
    UIGestureRecognizer *tapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(goBack:)];
    [(UITapGestureRecognizer *)tapRecognizer setNumberOfTapsRequired:2];
    [(UITapGestureRecognizer *)tapRecognizer setNumberOfTouchesRequired:1];
    [view addGestureRecognizer:tapRecognizer];
    [tapRecognizer release];
    
    UIGestureRecognizer *pinchRecognizer = [[UIPinchGestureRecognizer alloc] initWithTarget:self action:@selector(goBack:)];
    [view addGestureRecognizer:pinchRecognizer];
    [pinchRecognizer release];
}

//Image resizen (bij rotating)
- (void)resizeImage:(UIImageView *)image forRectangle:(CGRect)aRect {
    UIImageView *imageView = [[UIImageView alloc] initWithImage:[[[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[NSString stringWithFormat:@"%@_%i",[(Stap *)[stappen anyObject] toepassing].stapImageURL,image.tag] ofType:@"jpg"]] autorelease]];
    
    float width = imageView.bounds.size.width;
    float height = imageView.bounds.size.height;
    [imageView release];
    if (width>aRect.size.width) {
        width = aRect.size.width;
        height = image.frame.size.height/image.frame.size.width*width;
    }
    if (height>aRect.size.height) {
        height = aRect.size.height;
        width = image.frame.size.width/image.frame.size.height*height;
    }
    
    image.bounds = CGRectMake(0, 0, width, height);
}

//Image toevoegen aan scrollview
- (void)showPicture:(UIImageView *)image forRect:(CGRect)aRect animated:(BOOL)animated {
    [self resizeImage:image forRectangle:aRect];
    
    float oldWidth = image.frame.size.width;
    float oldHeight = image.frame.size.height;
    
    float width = oldImage.frame.size.width;
    float height = image.frame.size.height/image.frame.size.width*width;
    float xCenter = (image.tag-1) * scrollImage.frame.size.width + aRect.origin.x - width/2 + oldImage.frame.size.width/2;
    float yCenter = aRect.origin.y - height/2 + oldImage.frame.size.height/2;
    
    image.frame = CGRectMake(xCenter, yCenter, width, height);
    image.image = nil;
    image.alpha = 0;
    [scrollImage addSubview:image];
    
    [UIView beginAnimations:nil context:nil];
    [UIView setAnimationDuration:0.5];
    image.alpha = 1; 
    [UIView commitAnimations];
    
    float xCenterNew = (image.tag-1)*scrollImage.frame.size.width+aRect.size.width/2 - oldWidth/2;
    float yCenterNew = aRect.size.height/2 - oldHeight/2;
    
    if (animated) {
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationDuration:0.5];
        [UIView setAnimationDelay:0.5];
        image.frame = CGRectMake(xCenterNew, yCenterNew, oldWidth, oldHeight);
        [UIView commitAnimations];
    } else {
        image.frame = CGRectMake(xCenterNew, yCenterNew, oldWidth, oldHeight);
    }
    
    image.userInteractionEnabled = YES;
    [self addGestureRecognizer:image];
}

//Voorbereiding weergave eerste image
- (void)showFirstPicture {
    float superWidth = 1024;
    float superHeight = 748;
    float xPos = 412;
    float yPos = 244;
    if (UIInterfaceOrientationIsPortrait(self.interfaceOrientation)) {
        superWidth = 768;
        superHeight = 1004;
        xPos = 20;
        yPos = 465;
    }
    
    UIImage *img = [[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[NSString stringWithFormat:@"%@_%i", [self getStapForTag:oldImage.tag].toepassing.stapImageURL, oldImage.tag] ofType:@"jpg"]];
    UIImageView *image = [[UIImageView alloc] initWithImage:img];
    [img release];
    image.tag = oldImage.tag;
    
    scrollImage = [[UIScrollView alloc] initWithFrame:CGRectMake(0, 0, superWidth, superHeight)];
    scrollImage.delegate = self;
    scrollImage.contentSize = CGSizeMake([stappen count]*scrollImage.frame.size.width, scrollImage.frame.size.height);
    scrollImage.pagingEnabled = YES;
    scrollImage.showsHorizontalScrollIndicator = NO;
    [self.view addSubview:scrollImage];
    
    [self showPicture:image forRect:CGRectMake(xPos, yPos, superWidth, superHeight) animated:YES];
    [image release];
}

//Voorbereiding weergave andere images
- (void)showOtherPictures {
    float superWidth = 1024;
    float superHeight = 748;
    float xPos = 412;
    float yPos = 274;
    if (UIInterfaceOrientationIsPortrait(self.interfaceOrientation)) {
        superWidth = 768;
        superHeight = 1004;
        xPos = 20;
        yPos = 465;
    }
    
    for (Stap *stap in stappen) {
        if ([stap.stap intValue] != oldImage.tag) {
            UIImage *img = [[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[NSString stringWithFormat:@"%@_%i", stap.toepassing.stapImageURL, [stap.stap intValue]] ofType:@"jpg"]];
            UIImageView *image = [[UIImageView alloc] initWithImage:img];
            [img release];
            image.tag = [stap.stap intValue];
            
            image.frame = CGRectMake(0, 0, image.frame.size.width, image.frame.size.height);
            [self showPicture:image forRect:CGRectMake(xPos, yPos, superWidth, superHeight) animated:NO];
            [image release];
        }
    }
}

//Init: stappen en image doorgeven
- (id)initWithImageView:(UIImageView *)anOldImage andStappen:(NSSet *)deStappen {
    self = [super init];
    if (self) {
        self.view.backgroundColor = [UIColor colorWithRed:0 green:0 blue:0 alpha:0];
              
        [UIView beginAnimations:nil context:nil];
        [UIView setAnimationDuration:0.5];
        [UIView setAnimationDelay:0.5];
        self.view.backgroundColor = [UIColor colorWithRed:0 green:0 blue:0 alpha:1];
        [UIView commitAnimations];
        
        oldImage = [anOldImage retain];
        stappen = [deStappen retain];
        
        [self showFirstPicture];
        [self showOtherPictures];
        [scrollImage scrollRectToVisible:CGRectMake((oldImage.tag-1)*scrollImage.bounds.size.width, 0, scrollImage.bounds.size.width, scrollImage.bounds.size.height) animated:NO];
        [self showImages:oldImage.tag];
        
        if (UIInterfaceOrientationIsPortrait(self.interfaceOrientation)) {
            currentOrientation = [[NSString alloc] initWithString:@"Portrait"];
        } else {
            currentOrientation = [[NSString alloc] initWithString:@"Landscape"];
        }
        startOrientation = [currentOrientation retain];
    }
    return self;
}

//Gesture enkel op UIImage
- (BOOL)gestureRecognizerShouldBegin:(UIGestureRecognizer *)gestureRecognizer {
    if ([gestureRecognizer.view isKindOfClass:[UIImageView class]]) {
        return YES;
    }
    return NO;
}

//Rotate
- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    float superWidth = 1024;
    float superHeight = 748;
    float xPos = 412;
    float yPos = 274;
    
    int stap = scrollImage.contentOffset.x/scrollImage.frame.size.width;
    
    if (UIInterfaceOrientationIsPortrait(toInterfaceOrientation)) {
        superWidth = 768;
        superHeight = 1004;
        xPos = 20;
        yPos = 465;
        currentOrientation = @"Portrait";
    } else {
        currentOrientation = @"Landscape";
    }
    
    scrollImage.frame = CGRectMake(0, 0, superWidth, superHeight);
    scrollImage.contentSize = CGSizeMake([stappen count]*scrollImage.frame.size.width, scrollImage.frame.size.height);
    scrollImage.contentOffset = CGPointMake(stap*scrollImage.frame.size.width, 0);
    
    for (UIImageView *image in [[scrollImage.subviews copy] autorelease]) {
        if ([image isKindOfClass:[UIImageView class]]) {
            [self resizeImage:image forRectangle:CGRectMake(xPos, yPos, superWidth, superHeight)];
            
            float xCenterNew = (image.tag-1)*scrollImage.frame.size.width+superWidth/2;
            float yCenterNew = superHeight/2;
            image.center = CGPointMake(xCenterNew, yCenterNew);
        }
    }
}
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

//images tonen die zichtbaar moeten zijn tijdens scrolling (~lazy loading)
- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    int newPage = (scrollImage.contentOffset.x+scrollView.frame.size.width/2)/scrollImage.frame.size.width+1;
    if (newPage!=currentPage) {
        currentPage = scrollImage.contentOffset.x/scrollImage.frame.size.width+1;
        [self showImages:currentPage];
    }
}

- (void)dealloc {
    [scrollImage release];
    [stappen release];
    [oldImage release];
    [currentOrientation release];
    [startOrientation release];
    [super dealloc];
}

@end
