//
//  ToepassingView.m
//  Gyproc
//
//  Created by Bert Carrein on 26/04/11.
//  Copyright 2011 DMS. All rights reserved.
//

#import "ToepassingView.h"
#import <QuartzCore/QuartzCore.h>
#import "Brain.h"


@implementation ToepassingView

@synthesize toepassingLabel, imageView, toepassing;
@synthesize delegate;
@synthesize delegateStart;
@synthesize selected;
@synthesize backgroundSelected, backgroundNotSelected, backgroundHighlighted;

//Touch component: toepassing weergeven
- (void)hitView:(id)sender {
    [delegateStart kiesToepassing:toepassing];
    [delegate kiesToepassing:toepassing forOrientation:@"none"];
    
    for (ToepassingView *view in [[self.superview.subviews copy] autorelease]) {
        if ([view isKindOfClass:[ToepassingView class]]) {
            view.selected = NO;
        }
    }
    self.selected = YES;
}

//Init: backgrounds & inhoud
- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        //background
        backgroundSelected = [[UIImage imageNamed:@"toepassing-selected-bg.png"] retain];
        backgroundNotSelected = [[UIImage imageNamed:@"toepassing-notselected-bg.png"] retain];
        backgroundHighlighted = [[UIImage imageNamed:@"toepassing-hl-bg.png"] retain];
        
        //View van toepassing component in scrollview aanmaken
        toepassingLabel = [[UILabel alloc] initWithFrame:CGRectMake(12, 92, frame.size.width-22, 50)];
        toepassingLabel.backgroundColor = [UIColor clearColor];
        toepassingLabel.font = [UIFont systemFontOfSize:14];
        toepassingLabel.textColor = [UIColor colorWithRed:0.12 green:0.33 blue:0.6 alpha:1];
        toepassingLabel.numberOfLines = 2;
        [self addSubview:toepassingLabel];
        
        //Image init
        imageView = [[UIImageView alloc] initWithFrame:CGRectMake(12, 11, frame.size.width-24, 82)];
        imageView.layer.cornerRadius = 10;
        imageView.backgroundColor = [UIColor blackColor];
        imageView.clipsToBounds = YES;
        [self addSubview:imageView];
        
        video = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"toepassing-play.png"]];
        video.hidden = YES;
        video.frame = CGRectMake(imageView.frame.origin.x+imageView.frame.size.width-25, imageView.frame.origin.y+imageView.frame.size.height-20, 25, 27);
        [self addSubview:video];
        [video release];
        
        self.selected = NO;
        [self setBackgroundImage:backgroundSelected forState:UIControlStateDisabled];
        [self setBackgroundImage:backgroundNotSelected forState:UIControlStateNormal];
        [self setBackgroundImage:backgroundHighlighted forState:UIControlStateHighlighted];
        
        [self addTarget:self action:@selector(hitView:) forControlEvents:UIControlEventTouchUpInside];
    }
    return self;
}

//Toepassing view selected (witte achtergrond)
- (void)setSelected:(BOOL)aBool {
    selected = aBool;
    
    if (aBool) {
        self.enabled = NO;
        toepassingLabel.font = [UIFont boldSystemFontOfSize:14];
    } else {
        self.enabled = YES;
        toepassingLabel.font = [UIFont systemFontOfSize:14];
    }
    imageView.alpha = 1;
}

//FR video's bestaan nog niet
- (void)controleerVideoForLang:(NSString *)ln {
    if ([ln isEqualToString:@"NL"]) {
        if ([toepassing.videoURLNL isEqualToString:@"geen"]) {
            video.hidden = YES;
        } else
            video.hidden = NO;
    } else {
        video.hidden = YES;
    }
}

//Inhoud in toepassing component invullen
- (void)setToepassing:(Toepassing *)aToepassing {
    toepassing = [aToepassing retain];
    [self setLanguage];
    UIImage *img = [[UIImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:[NSString stringWithFormat:@"%@", toepassing.imageURL] ofType:@"jpg"]];
    imageView.image = img;
    [img release];
    
    if (delegate) {
        [self controleerVideoForLang:[Brain brain].currentLang];
    } else {
        [self controleerVideoForLang:[Brain brain].currentLang];
    }
}

//taal
- (void)setLanguage {
    if ([[Brain brain].currentLang isEqualToString:@"NL"]) {
        toepassingLabel.text = [NSString stringWithFormat:@"%@", toepassing.titelNL];
    } else {
        toepassingLabel.text = [NSString stringWithFormat:@"%@", toepassing.titelFR];
    }
}

- (void)dealloc {
    [toepassing release];
    [imageView release];
    [toepassingLabel release];
    [backgroundSelected release];
    [backgroundNotSelected release];
    [backgroundHighlighted release];
    [super dealloc];
}

@end
