//
//  GyprocAppDelegate.m
//  Gyproc
//
//  Created by Bert Carrein on 6/04/11.
//  Copyright 2011 DMS. All rights reserved.
//

#import "GyprocAppDelegate.h"

static const NSInteger kGANDispatchPeriodSec = 10;

@implementation GyprocAppDelegate

@synthesize window=_window;
@synthesize managedObjectContext=__managedObjectContext;
@synthesize managedObjectModel=__managedObjectModel;
@synthesize persistentStoreCoordinator=__persistentStoreCoordinator;

//Start applicatie
- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    //Google analytics
    [[GANTracker sharedTracker] startTrackerWithAccountID:@"UA-7024207-3" dispatchPeriod:kGANDispatchPeriodSec delegate:self];
    NSError *error;
    if (![[GANTracker sharedTracker] trackPageview:@"/App did launch" withError:&error]) {
        NSLog(@"GA Error Overall.h: viewDidLoad");
    }
    [error release];
    
    //Taal opvragen en een Brain object aanmaken (Brain zoudt alle data bij + de huidige taal)
    //Als er de default taal niet overeenkomt met ofwel fr of nl, en er geen taal van de vorige sessie is opgeslagen, TussenView Controller tonen. Anders al onmiddellijk PageController tonen
    NSUserDefaults *prefs = [NSUserDefaults standardUserDefaults];
    NSString *currentLang = [prefs objectForKey:@"lang"];
    if (currentLang == nil) {
        [[Brain brainForManagedObjectContext:[self managedObjectContext]] getDataAndFiles];
        
        if ([[[NSLocale preferredLanguages] objectAtIndex:0] isEqualToString:@"nl"]) {
            currentLang = @"NL";
        } else if ([[[NSLocale preferredLanguages] objectAtIndex:0] isEqualToString:@"fr"]) {
            currentLang = @"FR";
        } else {
            TussenView *view = [[TussenView alloc] init];
            view.managedObjectContext = [self managedObjectContext];
            self.window.rootViewController = view;
            [view release];
        }
    } else {
        [[Brain brainForManagedObjectContext:[self managedObjectContext]] getData];
    }
    
    if (currentLang != nil) {
        [[Brain brain] setCurrentLang:currentLang];
        
        PageController *page = [[PageController alloc] init];
        page.managedObjectContext = [self managedObjectContext];
        self.window.rootViewController = page;
        [page release];
    }
    
    [self.window makeKeyAndVisible];
    return YES;
}

//Google Analytics tracker
- (void)trackerDispatchDidComplete:(GANTracker *)tracker eventsDispatched:(NSUInteger)eventsDispatched eventsFailedDispatch:(NSUInteger)eventsFailedDispatch {
    //NSLog(@"events dispatched: %d, events failed: %d", eventsDispatched, eventsFailedDispatch);
}

- (void)applicationWillTerminate:(UIApplication *)application {
    [self saveContext];
}
- (void)dealloc {
    [[GANTracker sharedTracker] stopTracker];
    [_window release];
    [__managedObjectContext release];
    [__managedObjectModel release];
    [__persistentStoreCoordinator release];
    [navController release];
    [super dealloc];
}
- (void)saveContext {
    NSError *error = nil;
    NSManagedObjectContext *managedObjectContext = self.managedObjectContext;
    if (managedObjectContext != nil)
    {
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error])
        {
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        } 
    }
}

#pragma mark - Core Data stack

- (NSManagedObjectContext *)managedObjectContext {
    if (__managedObjectContext != nil)
    {
        return __managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil)
    {
        __managedObjectContext = [[NSManagedObjectContext alloc] init];
        [__managedObjectContext setPersistentStoreCoordinator:coordinator];
    }
    return __managedObjectContext;
}
- (NSManagedObjectModel *)managedObjectModel {
    if (__managedObjectModel != nil)
    {
        return __managedObjectModel;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"Gyproc" withExtension:@"momd"];
    __managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];    
    return __managedObjectModel;
}
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
    if (__persistentStoreCoordinator != nil)
    {
        return __persistentStoreCoordinator;
    }
    
    NSURL *storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"Gyproc.sqlite"];
    
    NSError *error = nil;
    __persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![__persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeURL options:nil error:&error])
    {
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }    
    
    return __persistentStoreCoordinator;
}

#pragma mark - Application's Documents directory

- (NSURL *)applicationDocumentsDirectory {
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}

@end
