//
//  StoreLocator.m
//  Gyproc
//
//  Created by Bert Carrein on 8/06/11.
//  Copyright 2011 DMS. All rights reserved.
//

#import "StoreLocator.h"
#import "StoreAnnotation.h"
#import "Reachability.h"
#import "Brain.h"


@implementation StoreLocator

@synthesize generalMapView;
@synthesize mapAnnotations;
@synthesize internetActive, hostActive;

#pragma mark - View lifecycle

//annotation toevoegen
- (void)addAnnotationsToMap {
    [self.generalMapView addAnnotations:mapAnnotations];
}

//Connectie testen
- (void)testConnection {
    if (!self.internetActive || !self.hostActive) {
        if ([[Brain brain].currentLang isEqualToString:@"NL"]) {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Geen verbinding" message:@"Er is geen internetverbinding beschikbaar om de kaarten in te laden." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
            [alert show];
            [alert release];
        } else {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Pas de connection" message:@"Er is geen internetverbinding beschikbaar om de kaarten in te laden." delegate:nil cancelButtonTitle:nil otherButtonTitles:@"OK", nil];
            [alert show];
            [alert release];
        }
        
    }
}

//Check network status
- (void)checkNetworkStatus:(NSNotification *)notice {
    NetworkStatus internetStatus = [internetReachable currentReachabilityStatus];
    switch (internetStatus) {
        case NotReachable: {
            self.internetActive = NO;
            break;
            
        }
        case ReachableViaWiFi: {
            self.internetActive = YES;
            break;
            
        }
        case ReachableViaWWAN: {
            self.internetActive = YES;
            break;
        }
    }
    
    NetworkStatus hostStatus = [hostReachable currentReachabilityStatus];
    switch (hostStatus) {
        case NotReachable: {
            self.hostActive = NO;
            break;
        }
        case ReachableViaWiFi: {
            self.hostActive = YES;
            break;
        }
        case ReachableViaWWAN: {
            self.hostActive = YES;
            break;
        }
    }
}

//Load: annotations toevoegen + userLocation weergeven
- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.mapAnnotations = [[NSMutableArray alloc] init];
    self.generalMapView.delegate = self;
    self.generalMapView.showsUserLocation = YES;
    
    StoreAnnotation *annotation1 = [[StoreAnnotation alloc] init];
    annotation1.storeTag = 1;
    annotation1.storeName = @"Van Maele";
    annotation1.storeSite = @"http://www.google.be/search?q=van+maele+ruddervoorde";
    annotation1.storeInfo = @"St.Elooisstraat 90, 8020 Ruddervoorde (050/27 73 86)";
    annotation1.latitude = [NSNumber numberWithDouble:51.099156];
    annotation1.longitude = [NSNumber numberWithDouble:3.213201];
    [self.mapAnnotations addObject:annotation1];
    [annotation1 release];
    
    StoreAnnotation *annotation2 = [[StoreAnnotation alloc] init];
    annotation2.storeTag = 2;
    annotation2.storeName = @"Verhelst Bouwmaterialen NV";
    annotation2.storeSite = @"http://www.google.be/search?q=verhelst+bouwmaterialen+brugge";
    annotation2.storeInfo = @"Vaartdijkstraat 30, 8200 Brugge (050/38 13 46)";
    annotation2.latitude = [NSNumber numberWithDouble:51.184481];
    annotation2.longitude = [NSNumber numberWithDouble:3.232963];
    [self.mapAnnotations addObject:annotation2];
    [annotation2 release];
    
    StoreAnnotation *annotation3 = [[StoreAnnotation alloc] init];
    annotation3.storeTag = 3;
    annotation3.storeName = @"Vandenbroucke Houthandel";
    annotation3.storeSite = @"http://www.google.be/search?q=vandenbroucke+houthandel+beernem";
    annotation3.storeInfo = @"Noendreef 1A, 8730 Beernem (050/78 84 95)";
    annotation3.latitude = [NSNumber numberWithDouble:51.11733];
    annotation3.longitude = [NSNumber numberWithDouble:3.321927];
    [self.mapAnnotations addObject:annotation3];
    [annotation3 release];
    
    StoreAnnotation *annotation4 = [[StoreAnnotation alloc] init];
    annotation4.storeTag = 4;
    annotation4.storeName = @"De Loof";
    annotation4.storeSite = @"http://www.google.be/search?q=de+loof+brugge";
    annotation4.storeInfo = @"Britse Kaai 4, 8000 Brugge (050/33 63 01)";
    annotation4.latitude = [NSNumber numberWithDouble:51.225808];
    annotation4.longitude = [NSNumber numberWithDouble:3.224251];
    [self.mapAnnotations addObject:annotation4];
    [annotation4 release];
    
    StoreAnnotation *annotation5 = [[StoreAnnotation alloc] init];
    annotation5.storeTag = 5;
    annotation5.storeName = @"Verhelst Bouwmaterialen NV";
    annotation5.storeSite = @"http://www.google.be/search?q=verhelst+bouwmaterialen+ichtegem";
    annotation5.storeInfo = @"Ringlaan 2, 8480 Ichtegem (051/58 01 91)";
    annotation5.latitude = [NSNumber numberWithDouble:51.103185];
    annotation5.longitude = [NSNumber numberWithDouble:3.026969];
    [self.mapAnnotations addObject:annotation5];
    [annotation5 release];
    
    StoreAnnotation *annotation6 = [[StoreAnnotation alloc] init];
    annotation6.storeTag = 6;
    annotation6.storeName = @"Callewaert";
    annotation6.storeSite = @"http://www.callewaert.be";
    annotation6.storeInfo = @"Ringlaan 18, 8520 Kuurne (056/36 43 10)";
    annotation6.latitude = [NSNumber numberWithDouble:50.848968];
    annotation6.longitude = [NSNumber numberWithDouble:3.261802];
    [self.mapAnnotations addObject:annotation6];
    [annotation6 release];
    
    StoreAnnotation *annotation7 = [[StoreAnnotation alloc] init];
    annotation7.storeTag = 7;
    annotation7.storeName = @"Texas Hout & Bouwmarkt NV";
    annotation7.storeSite = @"http://www.google.be/search?q=texas+hout+bouwmarkt+wevelgem";
    annotation7.storeInfo = @"Kortrijkstraat 313, 8560 Wevelgem (056/42 07 40)";
    annotation7.latitude = [NSNumber numberWithDouble:50.814951];
    annotation7.longitude = [NSNumber numberWithDouble:3.202558];
    [self.mapAnnotations addObject:annotation7];
    [annotation7 release];
    
    StoreAnnotation *annotation8 = [[StoreAnnotation alloc] init];
    annotation8.storeTag = 8;
    annotation8.storeName = @"Stichelbout";
    annotation8.storeSite = @"http://www.stichelbout.be";
    annotation8.storeInfo = @"Boulevard Industriel 94, 7700 Mouscron (056/85 52 52)";
    annotation8.latitude = [NSNumber numberWithDouble:50.749029];
    annotation8.longitude = [NSNumber numberWithDouble:3.246439];
    [self.mapAnnotations addObject:annotation8];
    [annotation8 release];
    
    StoreAnnotation *annotation9 = [[StoreAnnotation alloc] init];
    annotation9.storeTag = 9;
    annotation9.storeName = @"Verdonckt Houtmagazijn";
    annotation9.storeSite = @"http://www.houtmagazijnverdonckt.be";
    annotation9.storeInfo = @"Doorniksesteenweg 202, 8580 Avelgem (056/64 40 42)";
    annotation9.latitude = [NSNumber numberWithDouble:50.768331];
    annotation9.longitude = [NSNumber numberWithDouble:3.443549];
    [self.mapAnnotations addObject:annotation9];
    [annotation9 release];
    
    StoreAnnotation *annotation10 = [[StoreAnnotation alloc] init];
    annotation10.storeTag = 10;
    annotation10.storeName = @"Dewulf Bouwstock";
    annotation10.storeSite = @"http://www.bouwstockdewulf.be";
    annotation10.storeInfo = @"Monseigneur Catrystraat 64, 8800 Rumbeke (056/50 91 10)";
    annotation10.latitude = [NSNumber numberWithDouble:50.891719];
    annotation10.longitude = [NSNumber numberWithDouble:3.123293];
    [self.mapAnnotations addObject:annotation10];
    [annotation10 release];
    
    [self performSelector:@selector(addAnnotationsToMap) withObject:nil afterDelay:1.5];
    [self performSelector:@selector(testConnection) withObject:nil afterDelay:3];
    
    //Internet connectie?
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(checkNetworkStatus:) name:kReachabilityChangedNotification object:nil];
    
    internetReachable = [[Reachability reachabilityForInternetConnection] retain];
    [internetReachable startNotifier];
    
    hostReachable = [[Reachability reachabilityWithHostName: @"www.apple.com"] retain];
    [hostReachable startNotifier];
}

//Rotate
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}

//Na klikken op annotation adres enz weergeven: StoreAnnotation object
- (void)showDetails:(id)sender {
    if ([sender isKindOfClass:[UIButton class]]) {
        for (StoreAnnotation *ann in mapAnnotations) {
            if (ann.storeTag == [(UIButton *)sender tag]) {
                NSURL *url = [[NSURL alloc] initWithString:ann.storeSite];
                [[UIApplication sharedApplication] openURL:url];
                [url release];
            }
        }
    }
}

//Inzoomen op kaart
- (void)mapView:(MKMapView *)mv didAddAnnotationViews:(NSArray *)views {
    for(MKAnnotationView *annotationView in views) {
        if(annotationView.annotation == mv.userLocation) {
            MKCoordinateRegion region;
            MKCoordinateSpan span;
            
            span.latitudeDelta = 0.17;
            span.longitudeDelta = 0.17;
            
            region.span = span;
            region.center = CLLocationCoordinate2DMake(50.824624, 3.249545);//;mv.userLocation.location.coordinate;
            
            [mv setRegion:region animated:NO];
            [mv regionThatFits:region];
        }
    }
}

//Laten weten aan mapview hoe annotation zich moeten gedragen (delegate method)
- (MKAnnotationView *)mapView:(MKMapView *)mapView viewForAnnotation:(id<MKAnnotation>)annotation {
    if ([annotation isKindOfClass:[MKUserLocation class]]) {
        nil;
    }
    
    if ([annotation isKindOfClass:[StoreAnnotation class]]) {
        static NSString *annotationIdentifier = @"annotationIdentifier";
        MKPinAnnotationView *pinView = (MKPinAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:annotationIdentifier];
        
        if (!pinView) {
            MKPinAnnotationView *customPinView = [[[MKPinAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:annotationIdentifier] autorelease];
            customPinView.pinColor = MKPinAnnotationColorRed;
            customPinView.animatesDrop = YES;
            customPinView.canShowCallout = YES;
            
            UIButton *rightButton = [UIButton buttonWithType:UIButtonTypeDetailDisclosure];
            rightButton.tag = [(StoreAnnotation *)annotation storeTag];
            [rightButton addTarget:self action:@selector(showDetails:) forControlEvents:UIControlEventTouchUpInside];
            customPinView.rightCalloutAccessoryView = rightButton;
            
            return customPinView;
        } else {
            pinView.annotation = annotation;
        }
        return pinView;
    }
    
    return nil;
}

//Map type veranderen (standard, satellite, hybrid)
- (IBAction)changeMapView:(id)sender {
    if ([sender isKindOfClass:[UISegmentedControl class]]) {
        UISegmentedControl *control = [(UISegmentedControl *)sender retain];
        
        if (control.selectedSegmentIndex==0) {
            self.generalMapView.mapType = MKMapTypeStandard;
        } else if (control.selectedSegmentIndex==1) {
            self.generalMapView.mapType = MKMapTypeSatellite;
        } else if (control.selectedSegmentIndex==2) {
            self.generalMapView.mapType = MKMapTypeHybrid;
        }
        
        [control release];
    }
}

//Ga terug naar vorige controller
- (IBAction)goBack:(id)sender {
    [self dismissModalViewControllerAnimated:YES];
}

- (void)viewDidUnload {
    [super viewDidUnload];
    
    self.generalMapView = nil;
    self.mapAnnotations = nil;
}
- (void)dealloc {
    [generalMapView release];
    [mapAnnotations release];
    
    [super dealloc];
}

@end
